/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension;

import org.eclipse.jface.resource.ImageDescriptor;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceStatus;

/**
 * Interface for the GUI of constraints.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: B9C83F5BA9158F7F9F978DC6D75134DD
 */
public interface IConstraintUI {

	/**
	 * Action to take when trying to open the given constraint instance. <code>canOpen</code> should
	 * be called before in order to know if the status can be opened.
	 * 
	 * @return true if the constraint instance could indeed be open, false if it needs further
	 *         handling.
	 */
	public boolean openStatus(ConstraintInstance ci, IConstraintInstanceStatus s);

	/** True if the constraint instance can be open. */
	boolean canOpen(ConstraintInstance ci, IConstraintInstanceStatus s);

	/** Method to execute when a constraint instance gets outdated. */
	public void onOutdate(ConstraintInstance ci);

	/** Returns a short user-friendly description for the current status of the constraint instance. */
	public String getMessage(ConstraintInstance ci, IConstraintInstanceStatus s);

	/**
	 * Set to <code>true</code> if you want that a failure of your constraint be displayed as a
	 * warning rather than as an error.
	 */
	public boolean displayAsWarning();

	/** Gets a general description for the constraint type. */
	String getDescription();

	/** Returns the image descriptor to be used as icon image. */
	public ImageDescriptor getIconImageDescriptor();

	/**
	 * Way for the constraint to indicate that the verification is often time consuming.
	 * This triggers a few goodies:
	 * - wait 1s before triggering the constraint autocheck (if relevant)
	 * - display a warning popup in case of first trigger of the verification
	 * TODO(2703) make instead an event "onActivate" in order to allow customization. For instance
	 * one could check if Z3 is present or not.
	 */
	public boolean isTimeConsuming();

	/**
	 * True if this constraint shall be always activated (and therefore the user shall not be
	 * offered the possibility to control whether the constraint is active or not).
	 */
	public boolean shouldBeManuallyActivated();
}
