package org.fortiss.tooling.kernel.ui.service;

import java.util.Set;

import org.eclipse.jface.resource.ImageDescriptor;
import org.fortiss.tooling.kernel.extension.IConstraint;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceContainer;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceStatus;
import org.fortiss.tooling.kernel.service.IConstraintService;
import org.fortiss.tooling.kernel.ui.extension.IConstraintUI;
import org.fortiss.tooling.kernel.ui.internal.ConstraintUIService;

/**
 * GUI aspects of {@link IConstraintService}.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: A8B1FB41DE1F79D3BA503C914A986F80
 */
public interface IConstraintUIService {

	/** Returns the service instance. */
	public static IConstraintUIService getInstance() {
		return ConstraintUIService.getInstance();
	}

	/** Action to take when trying to open the (status of the given) constraint instance. */
	void openStatus(ConstraintInstance ci);

	/** True if the (status of the) given constraint instance can be open. */
	boolean canOpen(ConstraintInstance ci);

	/** Install the given constraint instance: install notifiers and triggers an "onOutdate" event. */
	void install(ConstraintInstance ci);

	/**
	 * True if a failure of the given constraint instance shall be displayed as a warning instead of
	 * as an error.
	 */
	boolean shallDisplayAsWarning(ConstraintInstance ci);

	/** Returns a description for the given constraint. */
	String getDescription(Class<? extends IConstraint> c);

	/** Returns a description for the given constraint instance. */
	String getDescription(ConstraintInstance ci);

	/** Returns the image descriptor to be used as icon image for the given constraint instance. */
	ImageDescriptor getIconImageDescriptor(ConstraintInstance c);

	/**
	 * Browse the elements in the context of the given container, adds constraints instances if
	 * relevant and install them.
	 */
	void addMissingConstraintInstances(IConstraintInstanceContainer cstrContainer);

	/**
	 * Returns the status of the given constraint instance, possibly returns <code>null</code> if
	 * the constraint instance is not activated.
	 */
	IConstraintInstanceStatus getStatus(ConstraintInstance ci);

	/** Returns a list of all available constraints. */
	Set<Class<? extends IConstraint>> getAllConstraints();

	/** Returns a list of constraints which should always be activated. */
	Set<Class<? extends IConstraint>> getAlwaysActivatedConstraints();

	/** Returns a list of all constraints which are activated for the given top element. */
	Set<Class<? extends IConstraint>> getActivatedConstraints(ITopLevelElement top);

	/**
	 * Activates the constraint <code>cstr</code> in the context of the given constraints container.
	 */
	void activate(Class<? extends IConstraint> cstr, IConstraintInstanceContainer cstrContainer);

	/**
	 * Deactivates the constraint <code>cstr</code> in the context of the given constraints
	 * container.
	 * The constraint container is necessary to retrieve the constraints whose markers need to be
	 * deactivated.
	 */
	void deactivate(Class<? extends IConstraint> cstr, IConstraintInstanceContainer cstrContainer);

	/**
	 * Registers the constraint UI part with the kernel and associates it with its non-UI
	 * counterpart. The non-UI counterpart should however be registered separately.
	 */
	void registerConstraintUI(Class<? extends IConstraintUI> cstrUI,
			Class<? extends IConstraint> cstr);
}
